@@ -18,7 +18,7 @@
 #   modify it under the same terms as Perl itself.
 #
 # REVISION
-#   $Id: DocBook.pm 4099 2009-02-25 06:37:06Z andrew $
+#   $Id: DocBook.pm 4101 2009-02-25 22:29:21Z andrew $
 #
 # TODO
 #   * get all the view_* methods outputting valid DocBook XML
@@ -32,18 +32,24 @@ require 5.004;
 use strict;
 
 use Pod::POM::View;
+use Pod::POM::Constants qw( :all );
 use base qw( Pod::POM::View );
 
 use Text::Wrap;
 use List::MoreUtils qw(firstidx);
+#use Clone; # cloning doesn't seem to work at the moment
 #use Data::Dumper; # for debugging
 
 use constant DEFAULT_ROOT_ELEMENT    => 'article';
 use constant DEFAULT_TOPSECT_ELEMENT => 'sect1';
 
-our $VERSION = '0.02'; # Don't forget to update the VERSION section in the POD!!!
 
-my $INIT_CAPS   = 0;
+# Don't forget to update the VERSION section in the POD!!!
+our $VERSION = '0.03';
+
+
+our $DEBUG   = 0 unless defined $DEBUG;
+
 my $XML_PROTECT = 0;
 my @OVER;
 my %topsect = ( book    => 'chapter',
@@ -53,6 +59,23 @@ my %topsect = ( book    => 'chapter',
 my @section = qw( part chapter sect1 sect2 sect3 sect4 sect5 );
 my $head1off = (firstidx { $_ eq 'sect1' } @section) - 1;
 
+my %dont_ucfirst = map { $_ =>1 } qw {
+	a an at as and are
+	but by 
+	ere
+	for from
+	in into is
+	of on onto or over
+	per
+	the to that than
+	until unto upon
+	via
+	with while whilst within without
+        de von
+};
+
+
+
 #------------------------------------------------------------------------
 # new(%options)
 #
@@ -64,7 +87,7 @@ my $head1off = (firstidx { $_ eq 'sect1' } @section) - 1;
 #  * title
 #  * author
 #  * extracttoptitle
-#  * initcaptitles
+#  * titlecasing
 #------------------------------------------------------------------------
 
 sub new {
@@ -99,6 +122,8 @@ sub new {
 sub view {
     my ($self, $type, $item) = @_;
 
+    DEBUG("view $type");
+
     if ($type =~ s/^seq_//) {
         return $item;
     }
@@ -132,7 +157,9 @@ sub view {
 sub view_pod {
     my ($self, $pod) = @_;
 
-    my ($root, $title, $author, $pubid, $sysid, $intsubset);
+    DEBUG("view_pod\n");
+    my ($root, $author, $pubid, $sysid, $intsubset);
+    my $title = "";
     my @content = $pod->content;
     my $version_msg = sprintf("<!-- Generated by %s %s using Pod:::POM %s -->\n",
                               __PACKAGE__, $VERSION, $Pod::POM::VERSION);
@@ -171,12 +198,74 @@ sub view_pod {
 }
 
 
+#------------------------------------------------------------------------
+# _title_case_text($self, $text, $casepreserve, $is_subsequent)
+#
+# Convert the case of words in a text string to "title case".  There are
+# a couple of implementations of this (Text::Autoformat and 
+# Text::Capitalize).  This is a fairly simple implementation.
+#
+# 
+#------------------------------------------------------------------------
+
+
+sub _title_case_text {
+    my ($self, $text, $casepreserve, $is_subsequent) = @_;
+
+    my @words = grep { $_ } split(/\s+/, $text);
+    foreach my $word (@words) {
+        next if $casepreserve->{lc $word};
+        $word = lc $word;
+        $word = ucfirst $word unless $dont_ucfirst{$word};
+    }
+    $words[0] = ucfirst $words[0] unless $is_subsequent or $casepreserve->{$words[0]};
+    my $newtext = join(" ", @words);
+    $text =~ s/(\S.*\S)/$newtext/s;
+    return $text;
+}
+
+
+sub _title_case_seq {
+    my ($self, $node, $casepreserve, $is_subsequent) = @_;
+
+    return unless ref $node;
+
+    $casepreserve ||= {};
+    if (!ref $casepreserve) {
+        $casepreserve = { map { lc($_) => 1 } split(/[\,\|\s]+/, $casepreserve) };
+    }
+    elsif (ref $casepreserve eq  'ARRAY') {
+        $casepreserve = { map { lc($_) => 1 } @$casepreserve };
+    }
+
+
+    $node = $$node;
+    if ($node->[CMD] =~ /^[BI]?$/) {
+        foreach ( @{$node->[CONTENT]} ) {
+            if (ref $_) {
+                $self->_title_case_seq($_, $casepreserve, $is_subsequent);
+            }
+            else {
+                $_ = $self->_title_case_text($_, $casepreserve, $is_subsequent);
+            }
+            $is_subsequent = 1;
+        }
+    }
+}
+
 sub _view_headn {
     my ($self, $head, $level) = @_;
+
+    DEBUG("view_head$level\n");
+
     my $sect  = $section[$level + (ref $self ? $self->{_head1off} : $head1off)];
-    $INIT_CAPS++;
-    my $title = $head->title->present($self);
-    $INIT_CAPS--;
+    my $title = $head->title;
+    if (ref $self and $self->{titlecasing}) {
+#        $title = clone($title);
+        $self->_title_case_seq($title, $self->{casepreserve});
+    }
+
+    $title = $title->present($self, "head$level");
     return "<$sect>\n" 
         . "<title>$title</title>\n\n"
         . $head->content->present($self)
@@ -219,6 +308,8 @@ sub view_over {
     my ($self, $over) = @_;
     my ($start, $end, $strip);
 
+    DEBUG("view_over");
+
     my $items = $over->item();
     return "" unless @$items;
 
@@ -257,6 +348,8 @@ sub view_over {
 sub view_item {
     my ($self, $item) = @_;
 
+    DEBUG("view_item");
+
     my $over  = ref $self ? $self->{ OVER } : \@OVER;
     my $title = $item->title();
     my $strip = $over->[-1];
@@ -336,6 +429,8 @@ sub view_seq_file {
     return "<filename>$text</filename>";
 }
 
+
+
 sub view_seq_space {
     my ($self, $text) = @_;
     $text =~ s/\s/&nbsp;/g;
@@ -458,8 +553,6 @@ my $gunk = '/#~:.?+=&%@!\-';
 my $punc = '.:!?\-;';
 my $any  = "${ltrs}${gunk}${punc}";
 
-my %stopword = map { $_ => 1 } qw( a the of and but );
-
 sub view_seq_text {
      my ($self, $text) = @_;
 
@@ -471,7 +564,7 @@ sub view_seq_text {
         }
      }
 
-     if ($text !~ s{
+     $text =~ s{
         \b                           # start at word boundary
          (                           # begin $1  {
            $urls     :               # need resource and a colon
@@ -490,22 +583,21 @@ sub view_seq_text {
              |                       # or else
                  $                   #   then end of the string
          )
-       }{<a href="$1">$1</a>}igox)
-     {
-         if ($INIT_CAPS) {
-             my @words = split(/\s+/, $text);
-             foreach my $word (@words) {
-                 $word = $stopword{lc $word} ? lc $word : ucfirst lc $word 
-             }
-             $words[0] = ucfirst $words[0];
-             $text = join(" ", @words);
-         }
-     }
+       }{<a href="$1">$1</a>}igox;
 
      return $text;
 }
 
 
+#------------------------------------------------------------------------
+# DEBUG(@msg)
+#------------------------------------------------------------------------
+
+sub DEBUG {
+    print STDERR "DEBUG: ", @_ if $DEBUG;
+}
+
+
 1;
 
 
@@ -591,12 +683,17 @@ is C<article> or C<chapter> if C<root> is C<book>
 if true then if the first C<=head1> is C<NAME> then its content is
 extracted as the title of the root element (default is true)
 
-=item C<initcaptitles>
+=item C<titlecasing>
 
 if true then title text is converted to initial caps format, i.e. all
 words are initial capped except for stopwords such as "a", "the", "and",
 "of", "on", etc (default is enabled)
 
+=item C<preservecase>
+
+list of words for which case should be preserved in titles.  The list may be an array ref or a
+string of words separated by spaces, commas or vertical bar characters.
+
 =back
 
 
@@ -653,7 +750,7 @@ The following methods are specializations of the methods in L<Pod::POM::View>:
 
 =item view_verbatim
 
-
+=item DEBUG
 
 =back
 
@@ -663,7 +760,7 @@ Andrew Ford, C<< <A.Ford@ford-mason.co.uk> >>
 
 =head1 VERSION
 
-This is version 0.02 of C<Pod::POM::View::DocBook>.  
+This is version 0.03 of C<Pod::POM::View::DocBook>.  
 
 
 =head1 BUGS AND LIMITATIONS
